@@ -57,7 +57,7 @@ class MidiHandler:	# Quisk calls this to make the Midi handler instance.
       btn = self.app.idName2Button[name]
     except:
       return
-    if btn.idName == 'PTT' and not self.conf.hot_key_ptt_toggle:
+    if btn.idName == 'PTT' and not self.conf.midi_ptt_toggle:
       btn.SetValue(True, True)
     else:
       btn.Shortcut(None, name)
@@ -69,7 +69,7 @@ class MidiHandler:	# Quisk calls this to make the Midi handler instance.
       return
     if hasattr(btn, "repeat_state"):	# This is a QuiskRepeatbutton
       btn.Shortcut(None, "_end_")
-    elif btn.idName == 'PTT' and not self.conf.hot_key_ptt_toggle:
+    elif btn.idName == 'PTT' and not self.conf.midi_ptt_toggle:
       btn.SetValue(False, True)
   def ControlKnob(self, name):
     if self.midi_message[2] == 64:	# Mid control
