@@ -198,7 +198,7 @@ struct sound_conf {
 	int overrange;		// count of ADC overrange (clip) for non-soundcard device
 	int latencyCapt;
 	int latencyPlay;
-	int interupts;
+	int interrupts;
 	char msg1[QUISK_SC_SIZE];
 	char err_msg[QUISK_SC_SIZE];
 	// These parameters are for the microphone:
@@ -247,6 +247,18 @@ struct QuiskWav {			// data to create a WAV or RAW audio file
     int fpPos;
 } ;
 
+// Remote Quisk control head and slave by Ben, AC2YD
+extern int remote_control_head;
+extern int remote_control_slave;
+int read_remote_radio_sound_socket(struct sound_dev* dev, complex double * cSamples, int nSamples);
+int read_remote_mic_sound_socket(struct sound_dev* dev, complex double * cSamples);
+void send_remote_radio_sound_socket(complex double * cSamples, int nSamples);
+void send_remote_mic_sound_socket(complex double * cSamples, int nSamples);
+extern PyObject * quisk_start_control_head_remote_sound(PyObject * self, PyObject * args);
+extern PyObject * quisk_stop_control_head_remote_sound(PyObject * self, PyObject * args);
+extern PyObject * quisk_start_remote_radio_remote_sound(PyObject * self, PyObject * args);
+extern PyObject * quisk_stop_remote_radio_remote_sound(PyObject * self, PyObject * args);
+
 void QuiskWavClose(struct QuiskWav *);
 int QuiskWavWriteOpen(struct QuiskWav *, char *, short, short, short, int, double);
 void QuiskWavWriteC(struct QuiskWav *, complex double *, int);
@@ -381,7 +393,7 @@ void quisk_hermes_tx_send(int, int *);
 void quisk_udp_mic_error(char *);
 void quisk_check_freedv_mode(void);
 void quisk_calc_audio_graph(double, complex double *, double *, int, int);
-int QuiskDeltaMsec(int);
+double QuiskDeltaSec(int);
 void * quisk_make_sidetone(struct sound_dev *, int);
 void * quisk_make_txIQ(struct sound_dev *, int);
 int quisk_play_sidetone(struct sound_dev *);
