@@ -227,6 +227,7 @@ class SliderBoxH:
     self.handler = handler
     self.display = display
     self.scale = scale
+    self.idName = text
     if display:		# Display the slider value
       t1 = self.text % (themin * scale)
       t2 = self.text % (themax * scale)
@@ -261,6 +262,8 @@ class SliderBoxH:
       event.Skip()
       if self.handler:
         self.handler(event)
+        if application.remote_control_head:
+          application.Hardware.RemoteCtlSend(f'{self.idName};{self.GetValue()}\n')
     if self.display:
       t = self.text % (self.slider.GetValue() * self.scale)
     else:
@@ -302,8 +305,9 @@ class SliderBoxV(wx.BoxSizer):
   def __init__(self, parent, text, init, themax, handler, display=False, themin=0):
     wx.BoxSizer.__init__(self, wx.VERTICAL)
     self.slider = wx.Slider(parent, -1, init, themin, themax, style=wx.SL_VERTICAL|wx.SL_INVERSE)
-    self.slider.Bind(wx.EVT_SCROLL, handler)
+    self.slider.Bind(wx.EVT_SCROLL, self.OnScroll)
     self.handler = handler
+    self.idName = text
     sw, sh = self.slider.GetSize()
     self.text = text
     self.themin = themin
@@ -337,6 +341,10 @@ class SliderBoxV(wx.BoxSizer):
     self.text_ctrl.SetForegroundColour(parent.GetForegroundColour())
     self.Add(self.text_ctrl, 0, 0)
     self.Add(self.slider, 1, 0)
+  def OnScroll(self, event):
+    self.handler(event)
+    if application.remote_control_head:
+      application.Hardware.RemoteCtlSend(f'{self.idName};{self.GetValue()}\n')
   def Change(self, event):
     event.Skip()
     self.text_ctrl.SetLabel(str(self.slider.GetValue()))
@@ -359,7 +367,7 @@ class SliderBoxV(wx.BoxSizer):
     value = int(value + 0.5)
     self.SetValue(value)
     if do_cmd:
-      self.handler(None)
+      self.OnScroll(None)
 
 class QuiskText1(wx.lib.stattext.GenStaticText):
   # Self-drawn text for QuiskText.
@@ -429,6 +437,8 @@ class QuiskText(wx.BoxSizer):
     self.Add(self.TextCtrl, 1, flag=wx.ALIGN_CENTER_VERTICAL)
   def SetLabel(self, label):
     self.TextCtrl.SetLabel(label)
+  def GetLabel(self):
+    return self.TextCtrl.GetLabel()
 
 # Start of our button classes.  They are compatible with wxPython GenButton
 # buttons.  Use the usual methods for access:
@@ -574,12 +584,19 @@ class QuiskPushbutton(QuiskButtons, wx.lib.buttons.GenButton):
   def OnButton(self, event):
     if self.command:
       self.command(event)
+      if application.remote_control_head:
+        application.Hardware.RemoteCtlSend(self.idName + ';1\n')
   def OnRightDown(self, event):
     self.direction = -1
     self.OnLeftDown(event) 
   def OnRightUp(self, event):
     self.OnLeftUp(event)
     self.direction = 1
+  def SetIndex(self, index):
+    if self.command and index:
+      event = wx.PyEvent()
+      event.SetEventObject(self)
+      self.command(event)
       
 
 class QuiskRepeatbutton(QuiskButtons, wx.lib.buttons.GenButton):
@@ -674,9 +691,18 @@ class QuiskCheckbutton(QuiskButtons, wx.lib.buttons.GenToggleButton):
       event = wx.PyEvent()
       event.SetEventObject(self)
       self.command(event)
+  def SetIndex(self, index, do_cmd=False):
+    self.SetValue(bool(index), do_cmd)
+  def GetIndex(self):
+    if self.GetValue():
+      return 1
+    else:
+      return 0
   def OnButton(self, event):
     if self.command:
       self.command(event)
+      if application.remote_control_head:
+        application.Hardware.RemoteCtlSend(f'{self.idName};{self.GetIndex()}\n')
   def OnRightDown(self, event):
     self.direction = -1
     self.OnLeftDown(event) 
@@ -896,6 +922,8 @@ class WrapControl(wx.BoxSizer):
     return self.button.GetValue()
   def SetValue(self, value, do_cmd=False):
     self.button.SetValue(value, do_cmd)
+  def SetIndex(self, index, do_cmd=False):
+    self.button.SetIndex(index, do_cmd)
   def GetLabel(self):
     return self.button.GetLabel()
   def __getattr__(self, name):
@@ -1005,6 +1033,8 @@ class WrapSlider(WrapControl):
       event = wx.PyEvent()
       event.SetEventObject(self.button)
       self.command(event)
+      if application.remote_control_head:
+        application.Hardware.RemoteCtlSend(f'{self.idName}Slider;{value}\n')
   def SetSlider(self, value=None, value_off=None, value_on=None):
     if value is not None:
       self.button.slider_value = value
@@ -1027,7 +1057,6 @@ class WrapDualSlider(WrapControl):	# Thanks to Steve, KF7O
     self.display = display                  # Display the value at the top
     WrapControl.__init__(self)
     self.Add(button, 1, flag=wx.ALIGN_CENTER_VERTICAL)
-    print("Size",self.button.GetSize())
     ## This is a hack to get _bitmap_sliderpop
     ## It would be better if _bitmap_sliderpop were not a global variable 
     ##but a first-class member in another module
@@ -1087,6 +1116,8 @@ class QuiskCycleCheckbutton(QuiskCheckbutton):
       event = wx.PyEvent()
       event.SetEventObject(self)
       self.command(event)
+  def GetIndex(self):
+    return self.index
   def OnButton(self, event):
     if not self.is_radio or self.button_down:
       self.direction = 1
@@ -1098,6 +1129,8 @@ class QuiskCycleCheckbutton(QuiskCheckbutton):
       self.direction = 0
     if self.command:
       self.command(event)
+      if application.remote_control_head:
+        application.Hardware.RemoteCtlSend(f'{self.idName};{self.GetIndex()}\n')
   def OnRightDown(self, event):		# Move left in the list of labels
     if not self.is_radio or self.GetValue():
       self.index -= 1
@@ -1107,6 +1140,8 @@ class QuiskCycleCheckbutton(QuiskCheckbutton):
       self.direction = -1
       if self.command:
         self.command(event)
+        if application.remote_control_head:
+          application.Hardware.RemoteCtlSend(f'{self.idName};{self.GetIndex()}\n')
   def OnLeftDclick(self, event):	# Left double-click: Set index zero
     if not self.is_radio or self.GetValue():
       self.index = 0
@@ -1114,6 +1149,8 @@ class QuiskCycleCheckbutton(QuiskCheckbutton):
       self.direction = 1
       if self.command:
         self.command(event)
+        if application.remote_control_head:
+          application.Hardware.RemoteCtlSend(f'{self.idName};{self.GetIndex()}\n')
   def DrawLabel(self, dc, width, height, dx=0, dy=0):
     QuiskCheckbutton.DrawLabel(self, dc, width, height, dx, dy)
     self.DrawGlyphCycle(dc, width, height)
@@ -1137,7 +1174,7 @@ class RadioButtonGroup:
   """
   def __init__(self, parent, command, labels, default, shortcuts=()):
     self.command = command
-    self.buttons = []
+    self.buttons = []	# This list of buttons can also contain WrapControls
     self.button = None
     self.shortcuts = list(shortcuts[:])
     self.last_shortcut = 0
@@ -1175,7 +1212,7 @@ class RadioButtonGroup:
     for b in self.buttons:
       if self.button is not None:
         b.SetValue(False)
-      elif isinstance(b, QuiskCycleCheckbutton):
+      elif isinstance(b, QuiskCycleCheckbutton) or (isinstance(b, WrapControl) and isinstance(b.button, QuiskCycleCheckbutton)):
         try:
           index = b.labels.index(label)
         except ValueError:
@@ -1186,7 +1223,10 @@ class RadioButtonGroup:
           self.button = b
           b.SetValue(True)
           if direction is not None:
-            b.direction = direction
+            if isinstance(b, WrapControl):
+              b.button.direction = direction
+            else:
+              b.direction = direction
       elif b.GetLabel() == label:
         b.SetValue(True)
         self.button = b
@@ -1214,7 +1254,7 @@ class RadioButtonGroup:
     return self.button.GetLabel()
   def GetSelectedButton(self):		# return the selected button
     return self.button
-  def GetIndex(self):	# Careful.  Some buttons are complex.
+  def GetIndex(self):	# Careful.  Some buttons are WrapControls.
     if not self.button:
       return None
     return self.buttons.index(self.button)
@@ -1457,3 +1497,41 @@ class FreqSetter(wx.TextCtrl):
     except:
       value = 7000
     return value
+
+class QuiskMenu(wx.Menu):
+  def __init__(self, menu_name):
+    wx.Menu.__init__(self)
+    self.menu_name = menu_name
+    self.id2data = {}
+    self.item_text2id = {}
+  def InitData(self, item, handler):
+    self.Bind(wx.EVT_MENU, self.Handler, item)
+    item_text = item.GetItemLabelText()
+    self.id2data[item.GetId()] = (item_text, handler)
+    self.item_text2id[item_text] = item.GetId()
+  def Append(self, item_text, handler):
+    item = wx.Menu.Append(self, -1, item_text)
+    self.InitData(item, handler)
+    return item
+  def AppendCheckItem(self, item_text, handler, is_checked=0):
+    item = wx.Menu.AppendCheckItem(self, -1, item_text)
+    item.Check(is_checked)
+    self.InitData(item, handler)
+    return item
+  def AppendRadioItem(self, item_text, handler, is_checked=0):
+    item = wx.Menu.AppendRadioItem(self, -1, item_text)
+    item.Check(is_checked)
+    self.InitData(item, handler)
+    return item
+  def Handler(self, event=None, nid=0):
+    if event:
+      nid = event.GetId()
+    item_text, handler = self.id2data[nid]
+    handler(event)
+    if application.remote_control_head:
+      menu_item = self.FindItemById(nid)
+      if menu_item.IsCheckable():
+        checked = menu_item.IsChecked()
+      else:
+        checked = 0
+      application.Hardware.RemoteCtlSend('MENU;%s;%s;%d\n' % (self.menu_name, item_text, int(checked)))
