@@ -7,6 +7,7 @@
 #include <math.h>
 #include <sys/time.h>
 #include <time.h>
+#include <errno.h>
 
 #ifdef MS_WINDOWS
 #include <winsock2.h>
@@ -14,7 +15,7 @@
 #else
 #include <sys/socket.h>
 #include <arpa/inet.h>
-#include <netinet/in.h>
+#include <netinet/ip.h>
 #endif
 
 #include "quisk.h"
@@ -64,6 +65,8 @@ static int radio_sound_nshorts;						// number of shorts (two bytes) to send
 static int radio_sound_mic_nshorts;					// number of shorts (two bytes) to receive
 int quisk_active_sidetone;		// quisk_active_sidetone == 0 No sidetone; == 1 sidetone from wasapi; == 2 sidetone from old logic
 int quisk_midi_cwkey;			// the CW key from MIDI NoteOn/NoteOff messages
+int remote_control_head;
+int remote_control_slave;
 
 struct sound_conf quisk_sound_state;	// Current sound status
 struct sound_conf * pt_quisk_sound_state = &quisk_sound_state;
@@ -847,7 +850,7 @@ int quisk_read_sound(void)	// Called from sound thread
 #endif
 
 	//QuiskPrintTime("quisk_read_sound start", 0);
-	quisk_sound_state.interupts++;
+	quisk_sound_state.interrupts++;
 	if (quisk_use_serial_port) {
 		quisk_poll_hardware_key();
 	}
@@ -912,7 +915,7 @@ int quisk_read_sound(void)	// Called from sound thread
 	}
 	else {
 		QuiskSleepMicrosec(5000);
-		nSamples = QuiskDeltaMsec(1) * quisk_sound_state.sample_rate / 1000;
+		nSamples = (int)(0.5 + QuiskDeltaSec(1) * quisk_sound_state.sample_rate);
 		if (nSamples > SAMP_BUFFER_SIZE / 2)
 			nSamples = SAMP_BUFFER_SIZE / 2;
 		for (i = 0; i < nSamples; i++)
@@ -945,13 +948,18 @@ int quisk_read_sound(void)	// Called from sound thread
 	snprintf(str80, 80, "  rx i/q process %d samples", nSamples);
 	QuiskPrintTime(str80, 0);
 #endif
+	// For control_head role in remote control operation, receive radio sound via UDP
+	if (remote_control_head)
+		nSamples = read_remote_radio_sound_socket(&quisk_Playback, cSamples, nSamples);
+	// For remote_radio role in remote control operation, send radio sound via UDP
+	if (remote_control_slave)
+		send_remote_radio_sound_socket(cSamples, nSamples);
 
 	is_DGT = rxMode == DGT_U || rxMode == DGT_L || rxMode == DGT_IQ || rxMode == DGT_FM;
 	if (quisk_record_state == PLAYBACK)
 		quisk_tmp_playback(cSamples, nSamples, 1.0);		// replace radio sound
 	else if (quisk_record_state == PLAY_FILE)
 		quisk_file_playback(cSamples, nSamples, 1.0);		// replace radio sound
-   
 	// Play the demodulated audio
 #if DEBUG_MIC != 2
 	if ( ! quisk_play_sidetone(&quisk_Playback))	// play sidetone
@@ -984,7 +992,7 @@ int quisk_read_sound(void)	// Called from sound thread
 	else if (radio_sound_mic_socket != INVALID_SOCKET)
 		mic_count = read_radio_sound_socket(cSamples);
 	else {      // No mic source; use zero samples
-		mic_count = QuiskDeltaMsec(0) * mic_sample_rate / 1000;
+		mic_count = (int)(0.5 + QuiskDeltaSec(0) * mic_sample_rate);
 		if (mic_count > SAMP_BUFFER_SIZE / 2)
 			mic_count = SAMP_BUFFER_SIZE / 2;
 		for (i = 0; i < mic_count; i++)
@@ -1016,6 +1024,13 @@ int quisk_read_sound(void)	// Called from sound thread
 	if (file_rec_mic.fp)
 		quisk_record_audio(&file_rec_mic, cSamples, mic_count);
 
+	// For remote_radio role in remote control operation, read mic sound via UDP; replace any mic sound from above.
+	if (remote_control_slave)
+		mic_count = read_remote_mic_sound_socket(&quisk_MicPlayback, cSamples);
+
+	// For control_head role in remote control operation, send mic sound via UDP
+	if (remote_control_head)
+		send_remote_mic_sound_socket(cSamples, mic_count);
 
 	if (mic_count > 0) {
 #if DEBUG_MIC == 3
@@ -1321,7 +1336,7 @@ void quisk_open_sound(void)	// Called from GUI thread
 	quisk_sound_state.write_error = 0;
 	quisk_sound_state.underrun_error = 0;
 	quisk_sound_state.mic_read_error = 0;
-	quisk_sound_state.interupts = 0;
+	quisk_sound_state.interrupts = 0;
 	quisk_sound_state.rate_min = quisk_sound_state.rate_max = -99;
 	quisk_sound_state.chan_min = quisk_sound_state.chan_max = -99;
 	quisk_sound_state.msg1[0] = 0;
@@ -1479,8 +1494,8 @@ void quisk_start_sound(void)	// Called from sound thread
 		quisk_sound_state.chan_min = Capture.chan_min;
 		quisk_sound_state.chan_max = Capture.chan_max;
 	}
-	QuiskDeltaMsec(0);	// Set timer to zero
-	QuiskDeltaMsec(1);
+	QuiskDeltaSec(0);	// Set timer to zero
+	QuiskDeltaSec(1);
 	quisk_set_play_state();
 	quisk_play_state = STARTING;
 }
@@ -1539,9 +1554,11 @@ PyObject * quisk_micplay_channels(PyObject * self, PyObject * args)	// Called fr
 
 PyObject * quisk_set_sparams(PyObject * self, PyObject * args, PyObject * keywds)
 {  /* Call with keyword arguments ONLY; change local parameters */
-	static char * kwlist[] = {"dc_remove_bw", "digital_output_level", NULL} ;
+	static char * kwlist[] = {"dc_remove_bw", "digital_output_level", "remote_control_slave", "remote_control_head",
+		NULL} ;
 
-	if (!PyArg_ParseTupleAndKeywords (args, keywds, "|id", kwlist, &dc_remove_bw, &digital_output_level))
+	if (!PyArg_ParseTupleAndKeywords (args, keywds, "|idii", kwlist, &dc_remove_bw, &digital_output_level,
+	&remote_control_slave, &remote_control_head))
 		return NULL;
 	Py_INCREF (Py_None);
 	return Py_None;
